c***** bicubic.for ***** version: 15 Jan 2009 *************************
c Bicubic interpolation of the gravity modeling residual in H-K space
c   to the finer mesh of the EARS stack parameter space. Uses Numerical
c   Recipes subroutines bcuint.for and bcucof.for. Note this
c   interpolation was adopted after the bilinear approach was found to
c   be inadequate when the residual min gets tight.
c***********************************************************************
      subroutine bicubic(k,l,rk,rl,gint)
      implicit none
c
c Arguments:
c
      integer k,l
      real*8 gint,rk,rl
c
c Common Blocks:
c
      integer nhh,nkk
      real*8 gres(40,40),prb(127)
      real*8 grmn,hhmn,hhmx,hmn,kkmn,kkmx,kmn,wtn
      common/GRAVWT/gres,prb,grmn,hhmn,hhmx,hmn,kkmn,kkmx,kmn,wtn,nhh,
     &    nkk
c
c Local Variables:
c
      real*8 y(4),y1(4),y12(4),y2(4)
c
      y(1)=gres(k,l)
      y(2)=gres(k+1,l)
      y(3)=gres(k+1,l+1)
      y(4)=gres(k,l+1)
      if(k.ne.1)then
        y1(1)=(gres(k+1,l)-gres(k-1,l))/2d0
        y1(4)=(gres(k+1,l+1)-gres(k-1,l+1))/2d0
      else
        y1(1)=gres(k+1,l)-gres(k,l)
        y1(4)=gres(k+1,l+1)-gres(k,l+1)
      endif
      if(k.ne.nhh-1)then
        y1(2)=(gres(k+2,l)-gres(k,l))/2d0
        y1(3)=(gres(k+2,l+1)-gres(k,l+1))/2d0
      else
        y1(2)=gres(k+1,l)-gres(k,l)
        y1(3)=gres(k+1,l+1)-gres(k,l+1)
      endif
      if(l.ne.1)then
        y2(1)=(gres(k,l+1)-gres(k,l-1))/2d0
        y2(2)=(gres(k+1,l+1)-gres(k+1,l-1))/2d0
      else
        y2(1)=gres(k,l+1)-gres(k,l)
        y2(2)=gres(k+1,l+1)-gres(k+1,l)
      endif
      if(l.ne.nkk-1)then
        y2(3)=(gres(k+1,l+2)-gres(k+1,l))/2d0
        y2(4)=(gres(k,l+2)-gres(k,l))/2d0
      else
        y2(3)=gres(k+1,l+1)-gres(k+1,l)
        y2(4)=gres(k,l+1)-gres(k,l)
      endif
      if(k.ne.1.and.l.ne.1)then
        y12(1)=(gres(k+1,l+1)-gres(k+1,l-1)-gres(k-1,l+1)+
     &      gres(k-1,l-1))/4d0
      else
        y12(1)=0d0
      endif
      if(k.ne.nhh-1.and.l.ne.1)then
        y12(2)=(gres(k+2,l+1)-gres(k+2,l-1)-gres(k,l+1)+gres(k,l-1))/4d0
      else
        y12(2)=0d0
      endif
      if(k.ne.nhh-1.and.l.ne.nkk-1)then
        y12(3)=(gres(k+2,l+2)-gres(k+2,l)-gres(k,l+2)+gres(k,l))/4d0
      else
        y12(3)=0d0
      endif
      if(k.ne.1.and.l.ne.nkk-1)then
        y12(4)=(gres(k+1,l+2)-gres(k+1,l)-gres(k-1,l+2)+gres(k-1,l))/4d0
      else
        y12(4)=0d0
      endif
c
      call bcuint(y,y1,y2,y12,0d0,1d0,0d0,1d0,rk,rl,gint)
c
      return
      end

      subroutine bcuint(y,y1,y2,y12,x1l,x1u,x2l,x2u,x1,x2,ansy)
c
      real*8 y(4),y1(4),y12(4),y2(4)
      real*8 ansy,ansy1,ansy2,x1,x1l,x1u,x2,x2l,x2u
      integer i
c
      real*8 t,u,c(4,4)
c
      call bcucof(y,y1,y2,y12,x1u-x1l,x2u-x2l,c)
      if(x1u.eq.x1l.or.x2u.eq.x2l)pause 'bad input in bcuint'
      t=(x1-x1l)/(x1u-x1l)
      u=(x2-x2l)/(x2u-x2l)
      ansy=0d0
      ansy2=0d0
      ansy1=0d0
      do i=4,1,-1
        ansy=t*ansy+((c(i,4)*u+c(i,3))*u+c(i,2))*u+c(i,1)
        ansy2=t*ansy2+(3d0*c(i,4)*u+2d0*c(i,3))*u+c(i,2)
        ansy1=u*ansy1+(3d0*c(4,i)*t+2d0*c(3,i))*t+c(2,i)
      enddo
      ansy1=ansy1/(x1u-x1l)
      ansy2=ansy2/(x2u-x2l)
c
      return
      end

      subroutine bcucof(y,y1,y2,y12,d1,d2,c)
c
      real*8 d1,d2,c(4,4),y(4),y1(4),y12(4),y2(4)
      integer i,j,k,l
c
      real*8 d1d2,xx,cl(16),wt(16,16),x(16)
c
      save wt
      data wt/ 1d0, 0d0,-3d0, 2d0, 0d0, 0d0, 0d0, 0d0,
     &        -3d0, 0d0, 9d0,-6d0, 2d0, 0d0,-6d0, 4d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,
     &         3d0, 0d0,-9d0, 6d0,-2d0, 0d0, 6d0,-4d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,
     &         0d0, 0d0, 9d0,-6d0, 0d0, 0d0,-6d0, 4d0,
     &         0d0, 0d0, 3d0,-2d0, 0d0, 0d0, 0d0, 0d0,
     &         0d0, 0d0,-9d0, 6d0, 0d0, 0d0, 6d0,-4d0,
     &         0d0, 0d0, 0d0, 0d0, 1d0, 0d0,-3d0, 2d0,
     &        -2d0, 0d0, 6d0,-4d0, 1d0, 0d0,-3d0, 2d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,
     &        -1d0, 0d0, 3d0,-2d0, 1d0, 0d0,-3d0, 2d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,
     &         0d0, 0d0,-3d0, 2d0, 0d0, 0d0, 3d0,-2d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 3d0,-2d0,
     &         0d0, 0d0,-6d0, 4d0, 0d0, 0d0, 3d0,-2d0,
     &         0d0, 1d0,-2d0, 1d0, 0d0, 0d0, 0d0, 0d0,
     &         0d0,-3d0, 6d0,-3d0, 0d0, 2d0,-4d0, 2d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,
     &         0d0, 3d0,-6d0, 3d0, 0d0,-2d0, 4d0,-2d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,
     &         0d0, 0d0,-3d0, 3d0, 0d0, 0d0, 2d0,-2d0,
     &         0d0, 0d0,-1d0, 1d0, 0d0, 0d0, 0d0, 0d0,
     &         0d0, 0d0, 3d0,-3d0, 0d0, 0d0,-2d0, 2d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 1d0,-2d0, 1d0,
     &         0d0,-2d0, 4d0,-2d0, 0d0, 1d0,-2d0, 1d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,
     &         0d0,-1d0, 2d0,-1d0, 0d0, 1d0,-2d0, 1d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,
     &         0d0, 0d0, 1d0,-1d0, 0d0, 0d0,-1d0, 1d0,
     &         0d0, 0d0, 0d0, 0d0, 0d0, 0d0,-1d0, 1d0,
     &         0d0, 0d0, 2d0,-2d0, 0d0, 0d0,-1d0, 1d0/
c
      d1d2=d1*d2
      do i=1,4
        x(i)=y(i)
        x(i+4)=y1(i)*d1
        x(i+8)=y2(i)*d2
        x(i+12)=y12(i)*d1d2
      enddo
c
      do i=1,16
        xx=0d0
        do k=1,16
          xx=xx+wt(i,k)*x(k)
        enddo
        cl(i)=xx
      enddo
c
      l=0
      do i=1,4
        do j=1,4
          l=l+1
          c(i,j)=cl(l)
        enddo
      enddo
c
      return
      end

