c constants.inc is an include file in which the constant-valued 
c   variables for the cpw_parper code collection are declared and set.
c
c General Constants:
c      A:     Radius of the Earth in m.
c      dtr:   Conversion from degrees to radians.
c      gacc:  Gravitational acceleration at the Earth's surface
c      pi:    Well, pi!
c
      real*8 A,dtr,gacc,gc,pi
      parameter(A=6.371d6,gc=6.67428d-11,pi=3.141592653589793d0)
      parameter(gacc=9.8d0,dtr=pi/1.8d2)
c
c Array declaration parameters:
c      mint:  Number of rows/cols in optimal interpolation matrices
c      ngi:   Number of rows in gravity grid
c      ngj:   Number of columns in gravity grid
c      ngm:   Sample size for mirrored gravity, thickness, Vp/Vs subgrids
c      ngmx:  Sample size for mirrored gravity, thickness, Vp/Vs full map grids
c      ngr:   Sample size for projected gravity subgrid
c      ngs:   Sample size for projected gravity full map grid
c      nrad:  Maximum number of distances in semivariance
c      nsta:  Maximum number of seismic sites to keep
c      nti:   Number of rows in Te grid
c      ntj:   Number of columns in Te grid
c      nxm:   2D FFT work array vector size
c
      integer mint,ngi,ngj,ngm,ngmx,ngr,ngs,nrad,nsta,nti,ntj,nxm
      parameter(mint=151,ngi=2911,ngj=1651,ngm=64,ngmx=1024)
      parameter(ngr=33,ngs=513,nrad=101)
      parameter(nsta=6000,nti=397,ntj=233)
      parameter(nxm=30*ngmx)
c
c Mesh constants:
c      del:   Averaged sampling interval for semivariance
c      dgm:   Grid mesh sampling for map-scale gravity comparison
c      dgr:   Grid mesh sampling for site-scale gravity comparison
c
      real*8 del,dgm,dgr
      parameter(del=2d4,dgm=1.1d4,dgr=9d3)
c
c Means, standard deviations of density model parameters for stochastic inversion:
c      dele_bar:   Expected elevation shift
c      dele_std:   Standard deviation
c      delg_bar:   Expected gravity shift
c      delg_std:   Standard deviation
c      dkdrho_bar: Expected del-rho/del-K
c      dkdrho_std: Standard deviation
c      drho_bar:   Expected Moho density contrast
c      drho_std:   Standard deviation
c      falpha_bar: Expected coefficient of thermal expansion scaling
c      falpha_std: Standard deviation
c      fsed_bar:   Expected coefficient of sed gravity scaling
c      fsed_std:   Standard deviation
c
       real*8 dele_bar,dele_std,delg_bar,delg_std,dkdrho_bar
       real*8 dkdrho_std0,drho_bar,drho_std0,falpha_bar,falpha_std0
       real*8 fsed_bar,fsed_std
       parameter(dele_bar=3d2,dele_std=1d3)                ! Rough estimate
       parameter(delg_bar=-1d2,delg_std=1d3)               ! Rough estimate
       parameter(dkdrho_bar=1.6d3,dkdrho_std0=2.85d0)      ! Rough estimate (x.025) from regression in Lowry & Pérez-Gussinyé (2011)
       parameter(drho_bar=3d2,drho_std0=6.9d-1)            ! From Ito and Simons (x.005; note nonstandard "data" error weighting)
       parameter(falpha_bar=1.167d0,falpha_std0=1.4d-3)    ! Rough estimate (Afonso et al) (x.01; note nonstandard "data" error weighting)
       parameter(fsed_bar=1d0,fsed_std=1d-2)               ! Rough estimate (x.1; note nonstandard "data" error weighting)
c
c Other constants:
c      cgv:   Multiplier for first order TSA gravity due to a surface
c      drdvr: Expected partial derivative drho/dK to convert Vp/Vs to mass
c      dro:   Expected Delta-rho at the Moho
c      ro0:   Surface density (for isostatic correction; see cintg)
c      ro1:   Density at 100 km (for isostatic correction; see cintg)
c      tol:   Tolerance for keeping singular values from SVD 
c
      real*8 cgv,drdvr,dro,ro0,ro1,tol
      parameter(cgv=1d5*2d0*pi*gc,drdvr=2d2,dro=2d2)
      parameter(ro0=2.62d3,ro1=3.345d3,tol=1d-8)

