c***** cintg.for ***** version: 20 Nov 2013 ****************************
c Function cintg generates the normalized premultiplier for
c   upward continuation of flexure of a 1-d density profile to
c   a gravity signal at the Earth's surface (the integral term in eqn A2
c   of Lowry & Smith 1994, without the 2*pi*G constant term to convert
c   to gravity. Hence, units are mks units of density). Here, we use the
c   averaged density profile from Christensen & Mooney (1995) for the
c   crust, ignore the Moho (that part of gravity is calculated
c   separately in getdrho), and treat the remainder from Moho to 100 km
c   using the ~100 km density from Ito & Simons.
c***********************************************************************
      real*8 function cintg(k2d,drho)
      implicit none
      include 'constants.inc'
c
c Arguments:
c
      real*8 drho,k2d
c
c Local variables:
c
      integer i
      real*8 grad(6),rho(6),zdep(6)
c
c Data variables:
c
      save zdep
      data (zdep(i),i=1,6)
     &   /0d0,
     &    1d4,
     &    2d4,
     &    3d4,
     &    4d4,
     &    1d5/
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Density profile:
      rho(1)=ro0
      rho(2)=2.68d3
      rho(3)=2.79d3
      rho(4)=2.98d3
      rho(5)=3.1d3
      rho(6)=ro1-drho
c Density gradients:
      do i=1,5
        grad(i)=(rho(i+1)-rho(i))/(zdep(i+1)-zdep(i))
      enddo
      grad(6)=0d0
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Calculate the wavenumber-dependent effective density contrast:
      if(k2d.eq.0d0)then
        cintg=ro1-ro0-drho
      else
        cintg=0d0
        do i=1,5
          if(grad(i).ne.-9999d0)then
c*****************************************************
c Gravity response for a constant velocity gradient: *
c*****************************************************
            cintg=cintg+grad(i)*
     &          (dexp(-k2d*zdep(i))-dexp(-k2d*zdep(i+1)))/k2d
          else
            cintg=cintg+(rho(i+1)-rho(i))*dexp(-k2d*zdep(i))
          endif
        enddo
      endif
c
      return
      end

