c***** estsvar.for **** version: 23 Apr 2019 ***************************
c  Estimate parameters of a semivariogram approximating the
c    observational semivariance.
c***********************************************************************
      subroutine estsvar(n0)
      implicit none
      include 'constants.inc'
c
c Arguments:
c
      integer n0
c
c
c Common Blocks:
c
      integer n,nl
      real*8 tc_err,tc_exp,vv_err,vv_exp
      common/EXPECT/tc_err,tc_exp,vv_err,vv_exp,n,nl
c
      character*10 site(nsta)
      integer neq(nsta),nmeas
      real*8 el(nsta),cred(nsta)
      real*8 tc(nsta),tce(nsta),vv(nsta),vve(nsta),xl(nsta),yl(nsta)
      common/INDATA/site,el,cred,tc,tce,vv,vve,xl,yl,neq,nmeas
c
      integer ns(nrad),nr
      real*8 rr(nrad),svtc(nrad),svvv(nrad),wtc(nrad),wtv(nrad)
      common/SEMVAR/svtc,svvv,rr,wtc,wtv,ns,nr
c
      real*8 aradc,aradv,sillc,sillv,varc0,varv0
      common/SMVPAR/aradc,aradv,sillc,sillv,varc0,varv0
c
c Function Variables:
c
      real*8 semv
c
c Local Variables:
c
      integer i,j,nit
      real*8 gmat(2,2),dely(2)
      real*8 aha,cwt,dar,det,dv0,dfdm1,dfdm2,vwt,ym,yt,yv
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Parameter change file:
      if(nl.ne.0)then
        if(site(nl)(6:10).eq.'     ')then
          open(unit=14,file='HK_Stacks/'//site(nl)(1:5)//'.delt',
     &        status='unknown',access='append')
        elseif(site(nl)(7:10).eq.'    ')then
          open(unit=14,file='HK_Stacks/'//site(nl)(1:6)//'.delt',
     &        status='unknown',access='append')
        elseif(site(nl)(8:10).eq.'   ')then
          open(unit=14,file='HK_Stacks/'//site(nl)(1:7)//'.delt',
     &        status='unknown',access='append')
        elseif(site(nl)(9:10).eq.'  ')then
          open(unit=14,file='HK_Stacks/'//site(nl)(1:8)//'.delt',
     &        status='unknown',access='append')
        elseif(site(nl)(10:10).eq.' ')then
          open(unit=14,file='HK_Stacks/'//site(nl)(1:9)//'.delt',
     &        status='unknown',access='append')
        else
          open(unit=14,file='HK_Stacks/'//site(nl)//'.delt',
     &        status='unknown',access='append')
        endif
      endif
      write(14,'(i6,f10.6,f11.8)')n0-1,dsqrt(varc0),dsqrt(varv0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Semivariance model initial guesses:
      varc0=svtc(1)
      varv0=svvv(1)
      aradc=3d5
      aradv=1d5
      sillc=0d0
      cwt=0d0
      sillv=0d0
      vwt=0d0
      do i=1,nr
        if(rr(i).gt.1d6)then
          sillc=sillc+wtc(i)*svtc(i)
          cwt=cwt+wtc(i)
          sillv=sillv+wtv(i)*svvv(i)
          vwt=vwt+wtv(i)
        endif
      enddo
      sillc=sillc/cwt
      sillv=sillv/vwt
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Iterative nonlinear inversion for Tc semivariogram:
      nit=0
1     nit=nit+1
c Reset Hilbert matrix:
        do j=1,2
          dely(j)=0d0
          do i=1,2
            gmat(i,j)=0d0
          enddo
        enddo
        do i=1,nr
c Inversion for spherical model (Gaussian process)
          aha=rr(i)/aradc
          if(aha.lt.1d0)then
            dfdm1=1d0-1.5d0*aha+5d-1*aha**3
            dfdm2=(sillc-varc0)*1.5d0*(aha**3-aha)/aradc
            ym=svtc(i)-varc0-(sillc-varc0)*(1.5d0*aha-5d-1*aha**3)
          else
            dfdm1=0d0
            dfdm2=0d0
            ym=svtc(i)-sillc
          endif
          gmat(1,1)=gmat(1,1)+wtc(i)*dfdm1*dfdm1
          gmat(2,1)=gmat(2,1)+wtc(i)*dfdm1*dfdm2
          gmat(2,2)=gmat(2,2)+wtc(i)*dfdm2*dfdm2
          dely(1)=dely(1)+wtc(i)*dfdm1*ym
          dely(2)=dely(2)+wtc(i)*dfdm2*ym
        enddo
        det=gmat(1,1)*gmat(2,2)-gmat(2,1)*gmat(2,1)
        dv0=(gmat(2,2)*dely(1)-gmat(2,1)*dely(2))/det
        dar=(-gmat(2,1)*dely(1)+gmat(1,1)*dely(2))/det
        varc0=dmin1(varc0+dv0,svtc(1))
        varc0=dmax1(varc0,svtc(1)/2d0)
        aradc=dmax1(aradc+dar,rr(2))
        aradc=dmin1(aradc,rr(nr-1))
      if(dabs(dar).gt.1d1.and.nit.le.40)goto1
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Iterative nonlinear inversion for Vp/Vs semivariogram:
      nit=0
2     nit=nit+1
c Reset Hilbert matrix:
        do j=1,2
          dely(j)=0d0
          do i=1,2
            gmat(i,j)=0d0
          enddo
        enddo
        do i=1,nr
c Inversion for spherical model (Gaussian process)
          aha=rr(i)/aradv
          if(aha.lt.1d0)then
            dfdm1=1d0-1.5d0*aha+5d-1*aha**3
            dfdm2=(sillv-varv0)*1.5d0*(aha**3-aha)/aradv
            ym=svvv(i)-varv0-(sillv-varv0)*(1.5d0*aha-5d-1*aha**3)
          else
            dfdm1=0d0
            dfdm2=0d0
            ym=svvv(i)-sillv
          endif
          gmat(1,1)=gmat(1,1)+wtv(i)*dfdm1*dfdm1
          gmat(2,1)=gmat(2,1)+wtv(i)*dfdm1*dfdm2
          gmat(2,2)=gmat(2,2)+wtv(i)*dfdm2*dfdm2
          dely(1)=dely(1)+wtv(i)*dfdm1*ym
          dely(2)=dely(2)+wtv(i)*dfdm2*ym
        enddo
        det=gmat(1,1)*gmat(2,2)-gmat(2,1)*gmat(2,1)
        dv0=(gmat(2,2)*dely(1)-gmat(2,1)*dely(2))/det
        dar=(-gmat(2,1)*dely(1)+gmat(1,1)*dely(2))/det
        varv0=dmin1(varv0+dv0,svvv(1))
        varv0=dmax1(varv0,svvv(1)/2d0)
        aradv=dmax1(aradv+dar,rr(2))
        aradv=dmin1(aradv,rr(nr-1))
      if(dabs(dar).gt.1d1.and.nit.le.40)goto2
c
c     open(7,file='Plots/semvmod.dat',form='formatted',
      open(7,file='Plots_DLS/semvmod.dat',form='formatted',
     &    status='unknown')
      rewind(7)
      do i=1,nr
        yt=semv(rr(i)/aradc,varc0,sillc)
        yv=semv(rr(i)/aradv,varv0,sillv)
        write(7,*)rr(i)/1d3,dsqrt(yt),dsqrt(yv)
      enddo
      close(7)
c
      if(n0.eq.0)then
c       open(7,file='Plots/semvmod0.dat',form='formatted',
        open(7,file='Plots_DLS/semvmod0.dat',form='formatted',
     &      status='unknown')
        rewind(7)
        do i=1,nr
          yt=semv(rr(i)/aradc,varc0,sillc)
          yv=semv(rr(i)/aradv,varv0,sillv)
          write(7,*)rr(i)/1d3,dsqrt(yt),dsqrt(yv)
        enddo
        close(7)
      endif
c
c     open(7,file='Plots/sigma.dat',form='formatted',
      open(7,file='Plots_DLS/sigma.dat',form='formatted',
     &    status='unknown',access='append')
      write(7,'(i6,f10.6,f11.8)')n0,dsqrt(varc0),dsqrt(varv0)
      close(7)
c
c Parameter change file:
      if(nl.ne.0)then
        write(14,'(i6,f10.6,f11.8)')n0,dsqrt(varc0),dsqrt(varv0)
        close(14)
      endif
c
      return
      end

