c***** improb.for **** version: 13 Jan 2014 ****************************
c  Determine which of the EARS site estimates of crustal thickness is
c    most improbable based on semivariance properties.
c***********************************************************************
      subroutine improb(vmx,n0)
      implicit none
      include 'constants.inc'
c
c Arguments:
c
      integer n0
      real*8 vmx
c
c Common Blocks:
c
      integer n,nl
      real*8 tc_err,tc_exp,vv_err,vv_exp
      common/EXPECT/tc_err,tc_exp,vv_err,vv_exp,n,nl
c
      character*10 site(nsta)
      integer neq(nsta),nmeas
      real*8 el(nsta),cred(nsta)
      real*8 tc(nsta),tce(nsta),vv(nsta),vve(nsta),xl(nsta),yl(nsta)
      common/INDATA/site,el,cred,tc,tce,vv,vve,xl,yl,neq,nmeas
c
      integer inr(mint),nnt
      real*8 rnr(mint)
      common/NEARST/rnr,inr,nnt
c
      integer revised(nsta),settled(nsta)
      common/REVISD/revised,settled
c
      real*8 aradc,aradv,sillc,sillv,varc0,varv0
      common/SMVPAR/aradc,aradv,sillc,sillv,varc0,varv0
c
      real*8 dcy(nsta),dsy(nsta)
      common/STORCS/dcy,dsy
c
c Function Variables:
c
      real*8 semv
c
c Local Variables:
c
      integer i,j,nn
      real*8 ccs,r,rmx,ss,va,wt,xn
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Use vmx/R to assign the optimal site for evaluation:
      nl=n
      n=0
      rmx=0d0
      do i=1,nnt-1
        if(revised(inr(i)).gt.0)then
          nn=idint(dfloat(n0)/dfloat(revised(inr(i))))
        else
          nn=101
        endif
        xn=0d0
        va=0d0
        do j=1,inr(i)-1
          ccs=dcy(j)*dcy(inr(i))+dsy(j)*dsy(inr(i))*dcos(xl(j)-
     &        xl(inr(i)))
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r=A*dacos(ccs)
          ss=semv(r/aradc,varc0,sillc)
          wt=(sillc-ss)/(tce(j)*tce(j))
          va=va+wt*((tc(inr(i))-tc(j))**2)/ss
          xn=xn+wt
        enddo
        do j=inr(i)+1,nmeas
          ccs=dcy(j)*dcy(inr(i))+dsy(j)*dsy(inr(i))*dcos(xl(j)-
     &        xl(inr(i)))
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r=A*dacos(ccs)
          ss=semv(r/aradc,varc0,sillc)
          wt=(sillc-ss)/(tce(j)*tce(j))
          va=va+wt*((tc(inr(i))-tc(j))**2)/ss
          xn=xn+wt
        enddo
        va=dsqrt(va/xn)
        if(va/(rnr(i)*rnr(i)).gt.rmx.and.nn.gt.100)then
          if(va.gt.3d0.and.revised(inr(i)).lt.3)then
            n=inr(i)
            vmx=va
            rmx=va/(rnr(i)*rnr(i))
          elseif(revised(inr(i)).lt.1)then
            n=inr(i)
            vmx=va
            rmx=va/(rnr(i)*rnr(i))
          endif
        endif
      enddo
c
      if(n.ne.0)then
        write(6,*)'Rad-weighted improbability: ',rmx,' (',dsqrt(vmx/
     &      rmx)/1d3,' km)'
        revised(n)=revised(n)+1
        return
      endif
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c If no sites tripped near the max grav residual, evaluate fractal 
c     violation at all sites relative to all other sites:
      vmx=-9d0
      do i=1,nmeas
        if(revised(i).gt.0)then
          nn=idint(dfloat(n0)/dfloat(revised(i)))
        else
          nn=101
        endif
        xn=0d0
        va=0d0
        do j=1,i-1
          ccs=dcy(j)*dcy(i)+dsy(j)*dsy(i)*dcos(xl(j)-xl(i))
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r=A*dacos(ccs)
          ss=semv(r/aradc,varc0,sillc)
          wt=(sillc-ss)/(tce(j)*tce(j))
          va=va+wt*((tc(i)-tc(j))**2)/ss
          xn=xn+wt
        enddo
        do j=i+1,nmeas
          ccs=dcy(j)*dcy(i)+dsy(j)*dsy(i)*dcos(xl(j)-xl(i))
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r=A*dacos(ccs)
          ss=semv(r/aradc,varc0,sillc)
          wt=(sillc-ss)/(tce(j)*tce(j))
          va=va+wt*((tc(i)-tc(j))**2)/ss
          xn=xn+wt
        enddo
        va=dsqrt(va/xn)
        if(va.gt.vmx.and.nn.gt.100)then
          if(va.gt.3d0.and.revised(i).lt.3)then
            n=i
            vmx=va
          elseif(revised(i).lt.1)then
            n=i
            vmx=va
          endif
        endif
      enddo
c
      if(n.ne.0)revised(n)=revised(n)+1
c
      return
      end

