c***** vamperr.for ***** version: 27 Mar 2018 **************************
c Improve EARS-derived crustal thickness and Vp/Vs estimates using 
c    chi-square cumulative distribution functions derived from gravity
c    modeling to re-weight H-K stack amplitudes.
c***********************************************************************
      program vamperr
      implicit none
      include 'constants.inc'
c
c Common Blocks:
c
      integer ncgg,ncvg
      real*8 rhgs(100),rhwg(100),rhws(100),rkgs(100),rkwg(100),rkws(100)
      real*8 maw(100),mag(100),mas(100)
      common/CVGSTT/maw,mag,mas,rhgs,rhwg,rhws,rkgs,rkwg,rkws,ncgg,ncvg
c
      real*8 delg,drho,dkdrho,falpha,ffwh,ffwk,ffwt,frav,fsed
      common/DENPAR/delg,drho,dkdrho,falpha,ffwh,ffwk,ffwt,frav,fsed
c
      integer n,nl
      real*8 tc_err,tc_exp,vv_err,vv_exp
      common/EXPECT/tc_err,tc_exp,vv_err,vv_exp,n,nl
c
      character*10 site(nsta)
      integer neq(nsta),nmeas
      real*8 el(nsta),cred(nsta)
      real*8 tc(nsta),tce(nsta),vv(nsta),vve(nsta),xl(nsta),yl(nsta)
      common/INDATA/site,el,cred,tc,tce,vv,vve,xl,yl,neq,nmeas
c
      integer revised(nsta),settled(nsta)
      common/REVISD/revised,settled
c
c Local Variables:
c
      character card*6,filen*11
      integer i,inn,iter,j,n0
      real*8 v,vmx
c
      call getarg(1,card)
      read(card,*)iter
      call getarg(2,card)
      read(card,*)inn
      n0=inn
      ncvg=0
      ncgg=0
      n=0
      nl=0
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Read in file data from files:
      call read_ears(iter,inn)
      call read_grav
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Convert crustal thicknesses to moho referenced to a sea level datum:
      call modatum
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Initialize using latest estimate of density parameters:
      drho=2.8d2
      dkdrho=1.5d3
      falpha=1d0
      fsed=1d0
c     open(unit=43,file='Plots/gresidual.dat',status='unknown')
      open(unit=43,file='Plots_DLS/gresidual.dat',status='unknown')
      rewind(43)
      if(inn.ne.0)then
1       read(43,'(i6,f8.2,f8.3,f8.2,f8.4)',end=2)j,v,drho,dkdrho,
     &      falpha
        goto3
2     endif
3     close(43)
c
c     open(unit=43,file='Plots/eresidual.dat',status='unknown')
      open(unit=43,file='Plots_DLS/eresidual.dat',status='unknown')
      rewind(43)
      if(inn.ne.0)then
4       read(43,'(i6,f8.4,f8.3,f8.2,f8.4)',end=5)j,v,ffwh,ffwk,ffwt
        goto6
5     endif
      ffwh=1d0
      ffwk=1d0
      ffwt=1d0
6     close(43)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Initial estimate of semivariance needed before kriging in getdrho:
      call semgram(n0)
      call estsvar(n0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Run one sweep:
      do j=iter,iter
        vmx=10d0
c
c Flag for whether site has been updated yet in this sweep:
        do i=1,nmeas
          revised(i)=0
        enddo
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Model the thermal mass variation for later removal from gravity:
c       call gettherm(n0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Model the best-fit map-scale gravity field from H-K estimates:
        call getdrho(n0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Rough estimate of unlikelihood to guide ordering of updates:
        call improb(vmx,n0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Iterate over sites:
        dowhile(vmx.gt.0d0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Update fix-valued sites:
          call settl(iter)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Estimate semivariogram from the data:
          call semgram(n0)
          call cogram(n0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Estimate semivariogram parameters from the observational data:
          call estsvar(n0)
          call estcvar(n0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Optimal interpolation to get expected value and uncertainty:
          call optint
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Use forward modeling of the gravity field to further narrow the range:
          call gravmod
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Update the EARS HK stack values using chi-square statistics and 
c   uncertainty estimates:
          call updstack(vmx,j,n0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Write updated files:
          call writ_ears
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Model the best-fit map-scale gravity field from H-K estimates:
          call getdrho(n0)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Rough estimate of unlikelihood to guide ordering of updates:
          call improb(vmx,n0)
c
        enddo
      enddo
c
      end

